#region License 
/**
	This is a NinjaScript implementation of the Ichimoku Clouds indicator.
    Copyright (C) 2011 Troder.wordpress.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/
#endregion

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Ichimoku Clouds Senkou Span
    /// </summary>
    [Description("Ichimoku Clouds Senkou Span")]
    public class IchimokuCloudsSenkouSpan : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int tenkanPeriod = 9;
		private int kijunPeriod = 26;
		private int senkouPeriod = 52;
		private int forward = 26;
		
		private Color senkouCloudAColor = Color.Green;
		private Color senkouCloudBColor = Color.Red;
		private int senkouCloudOpacity = 2;
		// User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Empty, PlotStyle.Line, "Tenkan-sen"));
			Add(new Plot(Color.Empty, PlotStyle.Line, "Kijun-sen"));
			Add(new Plot(SenkouCloudAColor, PlotStyle.Line, "Senkou Span A"));
			Add(new Plot(SenkouCloudBColor, PlotStyle.Line, "Senkou Span B"));
			
            Overlay	= true;
			//No plot until the max of all bars are available
			BarsRequired = System.Math.Max(System.Math.Max(tenkanPeriod,kijunPeriod),senkouPeriod);
			Displacement = Forward;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if(CurrentBars[0] <= BarsRequired)
				return;
			
			//Calculate plot values
			double vTenkanSen = (MAX(High,TenkanPeriod)[0] + MIN(Low,TenkanPeriod)[0])/2;
			double vKijunSen = (MAX(High,KijunPeriod)[0] + MIN(Low,KijunPeriod)[0])/2;
			double vSenkouSpanA = (vTenkanSen + vKijunSen)/2;
			double vSenkouSpanB = (MAX(High,SenkouPeriod)[0] + MIN(Low,SenkouPeriod)[0])/2;
			
			//Update plot values
			TenkanSen.Set(vTenkanSen);
			KijunSen.Set(vKijunSen);
			SenkouSpanA.Set(vSenkouSpanA);
			SenkouSpanB.Set(vSenkouSpanB);

			//Draw the clouds
			if(vSenkouSpanA > vSenkouSpanB)
			{
				DrawRegion(CurrentBar+"", -Forward-1, -Forward, SenkouSpanA, SenkouSpanB, Color.Empty, SenkouCloudAColor, SenkouCloudOpacity);
			}
			else
			{
				DrawRegion(CurrentBar+"", -Forward-1, -Forward, SenkouSpanA, SenkouSpanB, Color.Empty, SenkouCloudBColor, SenkouCloudOpacity);
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TenkanSen
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries KijunSen
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SenkouSpanA
        {
            get { return Values[2]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SenkouSpanB
        {
            get { return Values[3]; }
        }

        [Description("Tenkan/Conversion Line Period")]
        [GridCategory("Parameters")]
        public int TenkanPeriod
        {
            get { return tenkanPeriod; }
            set { tenkanPeriod = Math.Max(1, value); }
        }
		
		[Description("Kijun/Base Line Period")]
        [GridCategory("Parameters")]
        public int KijunPeriod
        {
            get { return kijunPeriod; }
            set { kijunPeriod = Math.Max(1, value); }
        }
		
		[Description("Senkou/Leading Span Period")]
        [GridCategory("Parameters")]
        public int SenkouPeriod
        {
            get { return senkouPeriod; }
            set { senkouPeriod = Math.Max(1, value); }
        }
		
		[Description("Senkou Cloud Color when Span A is higher")]
        [GridCategory("Parameters")]
        public Color SenkouCloudAColor
        {
            get { return senkouCloudAColor; }
            set { senkouCloudAColor = value; }
        }
		
		[Description("Senkou Cloud Color when Span B is higher")]
        [GridCategory("Parameters")]
        public Color SenkouCloudBColor
        {
            get { return senkouCloudBColor; }
            set { senkouCloudBColor = value; }
        }
		
		[Description("Senkou Cloud Opacity")]
        [GridCategory("Parameters")]
        public int SenkouCloudOpacity
        {
            get { return senkouCloudOpacity; }
            set { senkouCloudOpacity = Math.Max(0, value); }
        }
		
		[Description("Senkou Cloud Opacity")]
        [GridCategory("Parameters")]
        public int Forward
        {
            get { return forward; }
            set { forward = Math.Max(0, value); Displacement = value;}
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private IchimokuCloudsSenkouSpan[] cacheIchimokuCloudsSenkouSpan = null;

        private static IchimokuCloudsSenkouSpan checkIchimokuCloudsSenkouSpan = new IchimokuCloudsSenkouSpan();

        /// <summary>
        /// Ichimoku Clouds Senkou Span
        /// </summary>
        /// <returns></returns>
        public IchimokuCloudsSenkouSpan IchimokuCloudsSenkouSpan(int forward, int kijunPeriod, Color senkouCloudAColor, Color senkouCloudBColor, int senkouCloudOpacity, int senkouPeriod, int tenkanPeriod)
        {
            return IchimokuCloudsSenkouSpan(Input, forward, kijunPeriod, senkouCloudAColor, senkouCloudBColor, senkouCloudOpacity, senkouPeriod, tenkanPeriod);
        }

        /// <summary>
        /// Ichimoku Clouds Senkou Span
        /// </summary>
        /// <returns></returns>
        public IchimokuCloudsSenkouSpan IchimokuCloudsSenkouSpan(Data.IDataSeries input, int forward, int kijunPeriod, Color senkouCloudAColor, Color senkouCloudBColor, int senkouCloudOpacity, int senkouPeriod, int tenkanPeriod)
        {
            if (cacheIchimokuCloudsSenkouSpan != null)
                for (int idx = 0; idx < cacheIchimokuCloudsSenkouSpan.Length; idx++)
                    if (cacheIchimokuCloudsSenkouSpan[idx].Forward == forward && cacheIchimokuCloudsSenkouSpan[idx].KijunPeriod == kijunPeriod && cacheIchimokuCloudsSenkouSpan[idx].SenkouCloudAColor == senkouCloudAColor && cacheIchimokuCloudsSenkouSpan[idx].SenkouCloudBColor == senkouCloudBColor && cacheIchimokuCloudsSenkouSpan[idx].SenkouCloudOpacity == senkouCloudOpacity && cacheIchimokuCloudsSenkouSpan[idx].SenkouPeriod == senkouPeriod && cacheIchimokuCloudsSenkouSpan[idx].TenkanPeriod == tenkanPeriod && cacheIchimokuCloudsSenkouSpan[idx].EqualsInput(input))
                        return cacheIchimokuCloudsSenkouSpan[idx];

            lock (checkIchimokuCloudsSenkouSpan)
            {
                checkIchimokuCloudsSenkouSpan.Forward = forward;
                forward = checkIchimokuCloudsSenkouSpan.Forward;
                checkIchimokuCloudsSenkouSpan.KijunPeriod = kijunPeriod;
                kijunPeriod = checkIchimokuCloudsSenkouSpan.KijunPeriod;
                checkIchimokuCloudsSenkouSpan.SenkouCloudAColor = senkouCloudAColor;
                senkouCloudAColor = checkIchimokuCloudsSenkouSpan.SenkouCloudAColor;
                checkIchimokuCloudsSenkouSpan.SenkouCloudBColor = senkouCloudBColor;
                senkouCloudBColor = checkIchimokuCloudsSenkouSpan.SenkouCloudBColor;
                checkIchimokuCloudsSenkouSpan.SenkouCloudOpacity = senkouCloudOpacity;
                senkouCloudOpacity = checkIchimokuCloudsSenkouSpan.SenkouCloudOpacity;
                checkIchimokuCloudsSenkouSpan.SenkouPeriod = senkouPeriod;
                senkouPeriod = checkIchimokuCloudsSenkouSpan.SenkouPeriod;
                checkIchimokuCloudsSenkouSpan.TenkanPeriod = tenkanPeriod;
                tenkanPeriod = checkIchimokuCloudsSenkouSpan.TenkanPeriod;

                if (cacheIchimokuCloudsSenkouSpan != null)
                    for (int idx = 0; idx < cacheIchimokuCloudsSenkouSpan.Length; idx++)
                        if (cacheIchimokuCloudsSenkouSpan[idx].Forward == forward && cacheIchimokuCloudsSenkouSpan[idx].KijunPeriod == kijunPeriod && cacheIchimokuCloudsSenkouSpan[idx].SenkouCloudAColor == senkouCloudAColor && cacheIchimokuCloudsSenkouSpan[idx].SenkouCloudBColor == senkouCloudBColor && cacheIchimokuCloudsSenkouSpan[idx].SenkouCloudOpacity == senkouCloudOpacity && cacheIchimokuCloudsSenkouSpan[idx].SenkouPeriod == senkouPeriod && cacheIchimokuCloudsSenkouSpan[idx].TenkanPeriod == tenkanPeriod && cacheIchimokuCloudsSenkouSpan[idx].EqualsInput(input))
                            return cacheIchimokuCloudsSenkouSpan[idx];

                IchimokuCloudsSenkouSpan indicator = new IchimokuCloudsSenkouSpan();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Forward = forward;
                indicator.KijunPeriod = kijunPeriod;
                indicator.SenkouCloudAColor = senkouCloudAColor;
                indicator.SenkouCloudBColor = senkouCloudBColor;
                indicator.SenkouCloudOpacity = senkouCloudOpacity;
                indicator.SenkouPeriod = senkouPeriod;
                indicator.TenkanPeriod = tenkanPeriod;
                Indicators.Add(indicator);
                indicator.SetUp();

                IchimokuCloudsSenkouSpan[] tmp = new IchimokuCloudsSenkouSpan[cacheIchimokuCloudsSenkouSpan == null ? 1 : cacheIchimokuCloudsSenkouSpan.Length + 1];
                if (cacheIchimokuCloudsSenkouSpan != null)
                    cacheIchimokuCloudsSenkouSpan.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheIchimokuCloudsSenkouSpan = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Ichimoku Clouds Senkou Span
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IchimokuCloudsSenkouSpan IchimokuCloudsSenkouSpan(int forward, int kijunPeriod, Color senkouCloudAColor, Color senkouCloudBColor, int senkouCloudOpacity, int senkouPeriod, int tenkanPeriod)
        {
            return _indicator.IchimokuCloudsSenkouSpan(Input, forward, kijunPeriod, senkouCloudAColor, senkouCloudBColor, senkouCloudOpacity, senkouPeriod, tenkanPeriod);
        }

        /// <summary>
        /// Ichimoku Clouds Senkou Span
        /// </summary>
        /// <returns></returns>
        public Indicator.IchimokuCloudsSenkouSpan IchimokuCloudsSenkouSpan(Data.IDataSeries input, int forward, int kijunPeriod, Color senkouCloudAColor, Color senkouCloudBColor, int senkouCloudOpacity, int senkouPeriod, int tenkanPeriod)
        {
            return _indicator.IchimokuCloudsSenkouSpan(input, forward, kijunPeriod, senkouCloudAColor, senkouCloudBColor, senkouCloudOpacity, senkouPeriod, tenkanPeriod);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Ichimoku Clouds Senkou Span
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IchimokuCloudsSenkouSpan IchimokuCloudsSenkouSpan(int forward, int kijunPeriod, Color senkouCloudAColor, Color senkouCloudBColor, int senkouCloudOpacity, int senkouPeriod, int tenkanPeriod)
        {
            return _indicator.IchimokuCloudsSenkouSpan(Input, forward, kijunPeriod, senkouCloudAColor, senkouCloudBColor, senkouCloudOpacity, senkouPeriod, tenkanPeriod);
        }

        /// <summary>
        /// Ichimoku Clouds Senkou Span
        /// </summary>
        /// <returns></returns>
        public Indicator.IchimokuCloudsSenkouSpan IchimokuCloudsSenkouSpan(Data.IDataSeries input, int forward, int kijunPeriod, Color senkouCloudAColor, Color senkouCloudBColor, int senkouCloudOpacity, int senkouPeriod, int tenkanPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.IchimokuCloudsSenkouSpan(input, forward, kijunPeriod, senkouCloudAColor, senkouCloudBColor, senkouCloudOpacity, senkouPeriod, tenkanPeriod);
        }
    }
}
#endregion
